/* $Id: gffd.detect.c,v 1.3 2000/03/15 03:45:22 ajapted Exp $
******************************************************************************

   gffd.detect - file format detection utility

   Authors:  Andrew Apted   [andrew.apted@ggi-project.org]

   This software is placed in the public domain and can be used freely
   for any purpose. It comes without any kind of warranty, either
   expressed or implied, including, but not limited to the implied
   warranties of merchantability or fitness for a particular purpose.
   Use it at your own risk. the author is not responsible for any damage
   or consequences raised by use or inability to use this program.

******************************************************************************

   This program detects the file format of a file, and outputs the
   information on the standard output (stdout).

   USAGE:  gffd.detect  [ --batch]  [ --showall ]  [ --justname ]
                        [ --justmagic ]  filename  ...

******************************************************************************
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>

#include <gffd/gffd.h>


#define PROG_NAME  "gffd.detect"
#define PROG_VERS  "0.5"

#define BUFFER_SIZE  4096


static gffd_context_t my_ctx;

static unsigned char data_buf[BUFFER_SIZE];
static int data_len;

/* options */
static int batch_mode = 0;
static int show_all   = 0;
static int just_name  = 0;
static int just_magic = 0;


static void Usage(void)
{
  printf("\nUSAGE:  " PROG_NAME "  [OPTIONS]...  [FILE]...\n\n");

  printf(
"Available options:\n"
"  --help          Print a summary of the options\n"
"  --version       Print the version number\n"
"  --showall       Show all possible matches (not just the best)\n"
"  --justname      Match just the file's name, don't read any data\n"
"  --justmagic     Match just the file's data, ignore the filename\n"
"  --batch         Output is designed for other programs to parse\n\n");

  fflush(stdout);
}

static const char *StripPath(const char *filename)
{
  if (strrchr(filename, '/') == NULL) 
  {
    return filename;
  }

  return strrchr(filename, '/') + 1;
}

static int ReadFile(const char *filename)
{
  FILE *fp;

  /* open file, read data, and close the file */

  fp = fopen(filename, "rb");

  if (! fp)
  {
    fprintf(stderr, PROG_NAME ": error opening %s: %s\n",
        filename, strerror(errno));
    return -1;
  }

  data_len = fread(data_buf, 1, BUFFER_SIZE, fp);

  if (data_len < 0)
  {
    fprintf(stderr, PROG_NAME ": error reading %s: %s\n",
        filename, strerror(errno));
    fclose(fp);
    return -1;
  }

  fclose(fp);
  return 0;
}

static void HandleFile(const char *filename)
{
  int i, count;

  /* match the filename and file data */

  if (! just_name)
  {
    /* FIXME: check whether it is a directory */

    if (ReadFile(filename) < 0)
      return;
  }
  
  gffdBegin(my_ctx);

  if (! just_magic && filename[0] != '-')
    gffdMatchName(my_ctx, StripPath(filename));

  if (! just_name)
    gffdMatchData(my_ctx, data_buf, data_len);

  gffdPruneScore(my_ctx, 0);


  /* show results */

  count = gffdGetCount(my_ctx);

  if (! show_all && count > 1)
    count = 1;

  if (batch_mode)
    printf("%d\n", count);
  else
    printf("%s:\n", filename);

  if (count <= 0)
  {
    if (! batch_mode)
      printf("   0  data\n");
  }
  else
  {
    for (i=0; i < count; i++)
    {
      const char *MIME = gffdGetMatchMIME(my_ctx, i);

      printf("   %d  %s  %s", gffdGetMatchScore(my_ctx, i),
          gffdGetMatchClass(my_ctx, i), gffdGetMatchType(my_ctx, i));

      if (MIME)
        printf("  %s", MIME);
      else if (batch_mode)
        printf("  unknown");
      
      printf("\n");
    }
  }

  fflush(stdout);

  gffdEnd(my_ctx);
}


#define CHECK_OPT(option,pars)  \
    ((argc > (pars)) && (strcmp(argv[0],(option)) == 0))

#define SHUT_DOWN(err)  \
    do {  \
        gffdFreeContext(my_ctx);  \
        gffdExit();  \
        return (err);  \
    } while(0)


int main(int argc, char **argv, char **envp)
{
  /* initialize */

  if (gffdInit() < 0) 
  {
    fprintf(stderr, PROG_NAME ": initializing LibGFFD failed !\n");
    return 1;
  }

  my_ctx = gffdNewContext();


  /* handle all options and files */

  argv++; argc--;

  while (argc > 0)
  {
    if (CHECK_OPT("--batch", 0))
    {
      batch_mode = 1;
      argv++, argc--;
      continue;
    }

    if (CHECK_OPT("--showall", 0))
    {
      show_all = 1;
      argv++, argc--;
      continue;
    }

    if (CHECK_OPT("--justname", 0))
    {
      just_name = 1;
      argv++, argc--;
      continue;
    }

    if (CHECK_OPT("--justmagic", 0))
    {
      just_magic = 1;
      argv++, argc--;
      continue;
    }

    if (CHECK_OPT("-h", 0) || CHECK_OPT("--help", 0))
    {
      Usage();
      SHUT_DOWN(0);
    }

    if (CHECK_OPT("-v", 0) || CHECK_OPT("-V", 0) ||
        CHECK_OPT("--version", 0))
    {
      printf(PROG_NAME " version " PROG_VERS "\n");
      SHUT_DOWN(0);
    }

    /* FIXME: handle a single `-' meaning to read standard input */

    if (argv[0][0] == '-')
    {
      fprintf(stderr, PROG_NAME ": Unknown option `%s'\n", argv[0]);
      SHUT_DOWN(1);
    }

    HandleFile(argv[0]);

    argv++; argc--;
  }

  SHUT_DOWN(0);
}
